@@ -24,12 +24,11 @@ in case of connection problems or server API call overload.
 import logging
 
 from oslo_concurrency import lockutils
+from oslo_context import context
+from oslo_service import loopingcall
 from oslo_utils import excutils
-from oslo_utils import reflection
-import six
 
-from oslo_vmware._i18n import _, _LE, _LI, _LW
-from oslo_vmware.common import loopingcall
+from oslo_vmware._i18n import _
 from oslo_vmware import exceptions
 from oslo_vmware import pbm
 from oslo_vmware import vim
@@ -45,87 +44,7 @@ def _trunc_id(session_id):
         return session_id[-5:]
 
 
-# TODO(vbala) Move this class to excutils.py.
-class RetryDecorator(object):
-    """Decorator for retrying a function upon suggested exceptions.
-
-    The decorated function is retried for the given number of times, and the
-    sleep time between the retries is incremented until max sleep time is
-    reached. If the max retry count is set to -1, then the decorated function
-    is invoked indefinitely until an exception is thrown, and the caught
-    exception is not in the list of suggested exceptions.
-    """
-
-    def __init__(self, max_retry_count=-1, inc_sleep_time=10,
-                 max_sleep_time=60, exceptions=()):
-        """Configure the retry object using the input params.
-
-        :param max_retry_count: maximum number of times the given function must
-                                be retried when one of the input 'exceptions'
-                                is caught. When set to -1, it will be retried
-                                indefinitely until an exception is thrown
-                                and the caught exception is not in param
-                                exceptions.
-        :param inc_sleep_time: incremental time in seconds for sleep time
-                               between retries
-        :param max_sleep_time: max sleep time in seconds beyond which the sleep
-                               time will not be incremented using param
-                               inc_sleep_time. On reaching this threshold,
-                               max_sleep_time will be used as the sleep time.
-        :param exceptions: suggested exceptions for which the function must be
-                           retried
-        """
-        self._max_retry_count = max_retry_count
-        self._inc_sleep_time = inc_sleep_time
-        self._max_sleep_time = max_sleep_time
-        self._exceptions = exceptions
-        self._retry_count = 0
-        self._sleep_time = 0
-
-    def __call__(self, f):
-        func_name = reflection.get_callable_name(f)
-
-        def _func(*args, **kwargs):
-            result = None
-            try:
-                if self._retry_count:
-                    LOG.debug("Invoking %(func_name)s; retry count is "
-                              "%(retry_count)d.",
-                              {'func_name': func_name,
-                               'retry_count': self._retry_count})
-                result = f(*args, **kwargs)
-            except self._exceptions:
-                with excutils.save_and_reraise_exception() as ctxt:
-                    LOG.warning(_LW("Exception which is in the suggested list "
-                                    "of exceptions occurred while invoking "
-                                    "function: %s."),
-                                func_name,
-                                exc_info=True)
-                    if (self._max_retry_count != -1 and
-                            self._retry_count >= self._max_retry_count):
-                        LOG.error(_LE("Cannot retry upon suggested exception "
-                                      "since retry count (%(retry_count)d) "
-                                      "reached max retry count "
-                                      "(%(max_retry_count)d)."),
-                                  {'retry_count': self._retry_count,
-                                   'max_retry_count': self._max_retry_count})
-                    else:
-                        ctxt.reraise = False
-                        self._retry_count += 1
-                        self._sleep_time += self._inc_sleep_time
-                        return self._sleep_time
-            raise loopingcall.LoopingCallDone(result)
-
-        def func(*args, **kwargs):
-            loop = loopingcall.DynamicLoopingCall(_func, *args, **kwargs)
-            evt = loop.start(periodic_interval_max=self._max_sleep_time)
-            LOG.debug("Waiting for function %s to return.", func_name)
-            return evt.wait()
-
-        return func
-
-
-class VMwareAPISession(object):
+class VMwareAPISession:
     """Setup a session with the server and handles all calls made to it.
 
     Example:
@@ -139,7 +58,8 @@ class VMwareAPISession(object):
     def __init__(self, host, server_username, server_password,
                  api_retry_count, task_poll_interval, scheme='https',
                  create_session=True, wsdl_loc=None, pbm_wsdl_loc=None,
-                 port=443, cacert=None, insecure=True, pool_size=10):
+                 port=443, cacert=None, insecure=True, pool_size=10,
+                 connection_timeout=None, op_id_prefix='oslo.vmware'):
         """Initializes the API session with given parameters.
 
         :param host: ESX/VC server IP address or host name
@@ -161,6 +81,9 @@ class VMwareAPISession(object):
                          used only if cacert is not specified
         :param pool_size: Maximum number of connections in http
                           connection pool
+        :param connection_timeout: Maximum time in seconds to wait for peer to
+                                   respond.
+        :param op_id_prefix: String prefix for the operation ID.
         :raises: VimException, VimFaultException, VimAttributeException,
                  VimSessionOverLoadException
         """
@@ -180,13 +103,15 @@ class VMwareAPISession(object):
         self._cacert = cacert
         self._insecure = insecure
         self._pool_size = pool_size
+        self._connection_timeout = connection_timeout
+        self._op_id_prefix = op_id_prefix
         if create_session:
             self._create_session()
 
     def pbm_wsdl_loc_set(self, pbm_wsdl_loc):
         self._pbm_wsdl_loc = pbm_wsdl_loc
         self._pbm = None
-        LOG.info(_LI('PBM WSDL updated to %s'), pbm_wsdl_loc)
+        LOG.info('PBM WSDL updated to %s', pbm_wsdl_loc)
 
     @property
     def vim(self):
@@ -197,7 +122,9 @@ class VMwareAPISession(object):
                                 wsdl_url=self._vim_wsdl_loc,
                                 cacert=self._cacert,
                                 insecure=self._insecure,
-                                pool_maxsize=self._pool_size)
+                                pool_maxsize=self._pool_size,
+                                connection_timeout=self._connection_timeout,
+                                op_id_prefix=self._op_id_prefix)
         return self._vim
 
     @property
@@ -209,7 +136,9 @@ class VMwareAPISession(object):
                                 wsdl_url=self._pbm_wsdl_loc,
                                 cacert=self._cacert,
                                 insecure=self._insecure,
-                                pool_maxsize=self._pool_size)
+                                pool_maxsize=self._pool_size,
+                                connection_timeout=self._connection_timeout,
+                                op_id_prefix=self._op_id_prefix)
             if self._session_id:
                 # To handle the case where pbm property is accessed after
                 # session creation. If pbm property is accessed before session
@@ -217,7 +146,8 @@ class VMwareAPISession(object):
                 self._pbm.set_soap_cookie(self._vim.get_http_cookie())
         return self._pbm
 
-    @RetryDecorator(exceptions=(exceptions.VimConnectionException,))
+    @loopingcall.RetryDecorator(
+        exceptions=(exceptions.VimConnectionException,))
     @lockutils.synchronized('oslo_vmware_api_lock')
     def _create_session(self):
         """Establish session with the server."""
@@ -233,7 +163,8 @@ class VMwareAPISession(object):
         LOG.debug("Logging into host: %s.", self._host)
         session = self.vim.Login(session_manager,
                                  userName=self._server_username,
-                                 password=self._server_password)
+                                 password=self._server_password,
+                                 locale='en')
         self._session_id = session.key
         # We need to save the username in the session since we may need it
         # later to check active session. The SessionIsActive method requires
@@ -241,8 +172,8 @@ class VMwareAPISession(object):
         # object. We can't use the username used for login since the Login
         # method ignores the case.
         self._session_username = session.userName
-        LOG.info(_LI("Successfully established new session; session ID is "
-                     "%s."),
+        LOG.info("Successfully established new session; session ID is "
+                 "%s.",
                  _trunc_id(self._session_id))
 
         # Set PBM client cookie.
@@ -252,16 +183,16 @@ class VMwareAPISession(object):
     def logout(self):
         """Log out and terminate the current session."""
         if self._session_id:
-            LOG.info(_LI("Logging out and terminating the current session "
-                         "with ID = %s."),
+            LOG.info("Logging out and terminating the current session "
+                     "with ID = %s.",
                      _trunc_id(self._session_id))
             try:
                 self.vim.Logout(self.vim.service_content.sessionManager)
                 self._session_id = None
             except Exception:
-                LOG.exception(_LE("Error occurred while logging out and "
-                                  "terminating the current session with "
-                                  "ID = %s."),
+                LOG.exception("Error occurred while logging out and "
+                              "terminating the current session with "
+                              "ID = %s.",
                               _trunc_id(self._session_id))
         else:
             LOG.debug("No session exists to log out.")
@@ -282,9 +213,10 @@ class VMwareAPISession(object):
                  VimSessionOverLoadException, VimConnectionException
         """
 
-        @RetryDecorator(max_retry_count=self._api_retry_count,
-                        exceptions=(exceptions.VimSessionOverLoadException,
-                                    exceptions.VimConnectionException))
+        @loopingcall.RetryDecorator(
+            max_retry_count=self._api_retry_count,
+            exceptions=(exceptions.VimSessionOverLoadException,
+                        exceptions.VimConnectionException))
         def _invoke_api(module, method, *args, **kwargs):
             try:
                 api_method = getattr(module, method)
@@ -328,7 +260,7 @@ class VMwareAPISession(object):
                         fault = excep.fault_list[0]
                         clazz = exceptions.get_fault_class(fault)
                         if clazz:
-                            raise clazz(six.text_type(excep),
+                            raise clazz(str(excep),
                                         details=excep.details)
                     raise
 
@@ -364,7 +296,7 @@ class VMwareAPISession(object):
         except exceptions.VimException as ex:
             LOG.debug("Error: %(error)s occurred while checking whether the "
                       "current session: %(session)s is active.",
-                      {'error': six.text_type(ex),
+                      {'error': str(ex),
                        'session': _trunc_id(self._session_id)})
 
         return is_active
@@ -381,12 +313,13 @@ class VMwareAPISession(object):
         :raises: VimException, VimFaultException, VimAttributeException,
                  VimSessionOverLoadException, VimConnectionException
         """
-        loop = loopingcall.FixedIntervalLoopingCall(self._poll_task, task)
+        ctx = context.get_current()
+        loop = loopingcall.FixedIntervalLoopingCall(self._poll_task, task, ctx)
         evt = loop.start(self._task_poll_interval)
         LOG.debug("Waiting for the task: %s to complete.", task)
         return evt.wait()
 
-    def _poll_task(self, task):
+    def _poll_task(self, task, ctx):
         """Poll the given task until completion.
 
         If the task completes successfully, the method returns the task info
@@ -394,39 +327,47 @@ class VMwareAPISession(object):
         exception is set in the event.
 
         :param task: managed object reference of the task
+        :param ctx: request context for the corresponding task
         """
-        LOG.debug("Invoking VIM API to read info of task: %s.", task)
+        if ctx is not None:
+            ctx.update_store()
         try:
+            # we poll tasks too often, so skip logging the opID as it generates
+            # too much noise in the logs
             task_info = self.invoke_api(vim_util,
                                         'get_object_property',
                                         self.vim,
                                         task,
-                                        'info')
+                                        'info',
+                                        skip_op_id=True)
         except exceptions.VimException:
             with excutils.save_and_reraise_exception():
-                LOG.exception(_LE("Error occurred while reading info of "
-                                  "task: %s."),
+                LOG.exception("Error occurred while reading info of "
+                              "task: %s.",
                               task)
         else:
+            task_detail = {'id': vim_util.get_moref_value(task)}
+            # some internal tasks do not have 'name' set
+            if getattr(task_info, 'name', None):
+                task_detail['name'] = task_info.name
+
             if task_info.state in ['queued', 'running']:
                 if hasattr(task_info, 'progress'):
                     LOG.debug("Task: %(task)s progress is %(progress)s%%.",
-                              {'task': task,
+                              {'task': task_detail,
                                'progress': task_info.progress})
             elif task_info.state == 'success':
-                LOG.debug("Task: %s status is success.", task)
+                def get_completed_task():
+                    complete_time = getattr(task_info, 'completeTime', None)
+                    if complete_time:
+                        duration = complete_time - task_info.queueTime
+                        task_detail['duration_secs'] = duration.total_seconds()
+                    return task_detail
+                LOG.debug("Task: %s completed successfully.",
+                          get_completed_task())
                 raise loopingcall.LoopingCallDone(task_info)
             else:
-                error_msg = six.text_type(task_info.error.localizedMessage)
-                error = task_info.error
-                name = error.fault.__class__.__name__
-                fault_class = exceptions.get_fault_class(name)
-                if fault_class:
-                    task_ex = fault_class(error_msg)
-                else:
-                    task_ex = exceptions.VimFaultException([name],
-                                                           error_msg)
-                raise task_ex
+                raise exceptions.translate_fault(task_info.error)
 
     def wait_for_lease_ready(self, lease):
         """Waits for the given lease to be ready.
@@ -451,17 +392,17 @@ class VMwareAPISession(object):
 
         :param lease: lease whose state is to be polled
         """
-        LOG.debug("Invoking VIM API to read state of lease: %s.", lease)
         try:
             state = self.invoke_api(vim_util,
                                     'get_object_property',
                                     self.vim,
                                     lease,
-                                    'state')
+                                    'state',
+                                    skip_op_id=True)
         except exceptions.VimException:
             with excutils.save_and_reraise_exception():
-                LOG.exception(_LE("Error occurred while checking "
-                                  "state of lease: %s."),
+                LOG.exception("Error occurred while checking "
+                              "state of lease: %s.",
                               lease)
         else:
             if state == 'ready':
@@ -477,7 +418,7 @@ class VMwareAPISession(object):
                               "%(error_msg)s.") % {'lease': lease,
                                                    'error_msg': error_msg}
                 LOG.error(excep_msg)
-                raise exceptions.VimException(excep_msg)
+                raise exceptions.translate_fault(error_msg, excep_msg)
             else:
                 # unknown state
                 excep_msg = _("Unknown state: %(state)s for lease: "
@@ -495,8 +436,8 @@ class VMwareAPISession(object):
                                    lease,
                                    'error')
         except exceptions.VimException:
-            LOG.warning(_LW("Error occurred while reading error message for "
-                            "lease: %s."),
+            LOG.warning("Error occurred while reading error message for "
+                        "lease: %s.",
                         lease,
                         exc_info=True)
             return "Unknown"
