@@ -13,12 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Unit tests for functions and classes for image transfer.
-"""
+"""Unit tests for functions and classes for image transfer."""
 
-import mock
-import six
+import io
+from unittest import mock
 
 from oslo_vmware import exceptions
 from oslo_vmware import image_transfer
@@ -30,9 +28,9 @@ class ImageTransferUtilityTest(base.TestCase):
 
     def test_start_transfer(self):
         data = b'image-data-here'
-        read_handle = six.BytesIO(data)
+        read_handle = io.BytesIO(data)
         write_handle = mock.Mock()
-        image_transfer._start_transfer(read_handle, write_handle, None)
+        image_transfer._start_transfer(read_handle, write_handle, 10)
         write_handle.write.assert_called_once_with(data)
 
     @mock.patch('oslo_vmware.rw_handles.FileWriteHandle')
@@ -315,12 +313,16 @@ class ImageTransferUtilityTest(base.TestCase):
 
     @mock.patch.object(image_transfer, '_start_transfer')
     @mock.patch('oslo_vmware.rw_handles.VmdkReadHandle')
+    @mock.patch('oslo_service.loopingcall.FixedIntervalLoopingCall')
     def test_copy_stream_optimized_disk(
-            self, vmdk_read_handle, start_transfer):
+            self, loopingcall, vmdk_read_handle, start_transfer):
 
-        read_handle = mock.sentinel.read_handle
+        read_handle = mock.Mock()
         vmdk_read_handle.return_value = read_handle
 
+        updater = mock.Mock()
+        loopingcall.return_value = updater
+
         context = mock.sentinel.context
         timeout = mock.sentinel.timeout
         write_handle = mock.Mock(name='/cinder/images/tmpAbcd.vmdk')
@@ -338,8 +340,12 @@ class ImageTransferUtilityTest(base.TestCase):
 
         vmdk_read_handle.assert_called_once_with(
             session, host, port, vm, vmdk_file_path, vmdk_size)
+        loopingcall.assert_called_once_with(read_handle.update_progress)
+        updater.start.assert_called_once_with(
+            interval=image_transfer.NFC_LEASE_UPDATE_PERIOD)
         start_transfer.assert_called_once_with(read_handle, write_handle,
                                                timeout)
+        updater.stop.assert_called_once_with()
 
     @mock.patch('oslo_vmware.rw_handles.VmdkReadHandle')
     @mock.patch.object(image_transfer, '_start_transfer')
@@ -364,6 +370,8 @@ class ImageTransferUtilityTest(base.TestCase):
 
         image_name = 'fake_image'
         image_version = 1
+        store_id = 'fake_store'
+        base_image_ref = 'e79161cd-5f9d-4007-8823-81a807a64332'
 
         fake_VmdkReadHandle = mock.Mock()
         fake_rw_handles_VmdkReadHandle.return_value = fake_VmdkReadHandle
@@ -381,7 +389,9 @@ class ImageTransferUtilityTest(base.TestCase):
                                     vmdk_size=image_size,
                                     is_public=is_public,
                                     image_name=image_name,
-                                    image_version=image_version)
+                                    image_version=image_version,
+                                    store_id=store_id,
+                                    base_image_ref=base_image_ref)
 
         fake_rw_handles_VmdkReadHandle.assert_called_once_with(session,
                                                                host,
@@ -390,14 +400,17 @@ class ImageTransferUtilityTest(base.TestCase):
                                                                file_path,
                                                                image_size)
 
-        ver_str = six.text_type(image_version)
+        ver_str = str(image_version)
         image_metadata = {'disk_format': 'vmdk',
                           'name': image_name,
                           'properties': {'vmware_image_version': ver_str,
                                          'vmware_disktype': 'streamOptimized',
                                          'owner_id': owner_id}}
 
-        image_service.update.assert_called_once_with(context,
-                                                     image_id,
-                                                     image_metadata,
-                                                     data=fake_VmdkReadHandle)
+        image_service.update.assert_called_once_with(
+            context,
+            image_id,
+            image_metadata,
+            data=fake_VmdkReadHandle,
+            store_id=store_id,
+            base_image_ref=base_image_ref)
