@@ -13,15 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Unit tests for read and write handles for image transfer.
-"""
+"""Unit tests for read and write handles for image transfer."""
 
 import ssl
+from unittest import mock
 
-import mock
 import requests
-import six
 
 from oslo_vmware import exceptions
 from oslo_vmware import rw_handles
@@ -100,7 +97,7 @@ class FileWriteHandleTest(base.TestCase):
     """Tests for FileWriteHandle."""
 
     def setUp(self):
-        super(FileWriteHandleTest, self).setUp()
+        super().setUp()
 
         vim_cookie = mock.Mock()
         vim_cookie.name = 'name'
@@ -168,12 +165,30 @@ class VmdkHandleTest(base.TestCase):
 
         self.assertRaises(exceptions.VimException, handle.update_progress)
 
+    def test_fileno(self):
+        session = mock.Mock()
+        handle = rw_handles.VmdkHandle(session, None, 'fake-url', None)
+
+        self.assertRaises(IOError, handle.fileno)
+
+    def test_release_lease_incomplete_transfer(self):
+        session = mock.Mock()
+        handle = rw_handles.VmdkHandle(session, None, 'fake-url', None)
+
+        handle._get_progress = mock.Mock(return_value=99)
+        session.invoke_api = mock.Mock()
+        handle._release_lease()
+
+        session.invoke_api.assert_called_with(handle._session.vim,
+                                              'HttpNfcLeaseAbort',
+                                              handle._lease)
+
 
 class VmdkWriteHandleTest(base.TestCase):
     """Tests for VmdkWriteHandle."""
 
     def setUp(self):
-        super(VmdkWriteHandleTest, self).setUp()
+        super().setUp()
         self._conn = mock.Mock()
         patcher = mock.patch(
             'urllib3.connection.HTTPConnection')
@@ -202,7 +217,7 @@ class VmdkWriteHandleTest(base.TestCase):
         vim_cookie = mock.Mock()
         vim_cookie.name = 'name'
         vim_cookie.value = 'value'
-        session.vim.client.options.transport.cookiejar = [vim_cookie]
+        session.vim.client.cookiejar = [vim_cookie]
         return session
 
     def test_init_failure(self):
@@ -228,6 +243,16 @@ class VmdkWriteHandleTest(base.TestCase):
         self._conn.putrequest.assert_called_once_with('PUT', '/ds/disk1.vmdk')
         self._conn.send.assert_called_once_with(data)
 
+    def test_tell(self):
+        session = self._create_mock_session()
+        handle = rw_handles.VmdkWriteHandle(session, '10.1.2.3', 443,
+                                            'rp-1', 'folder-1', None,
+                                            100)
+        data = [1] * 10
+        handle.write(data)
+        self.assertEqual(len(data), handle._bytes_written)
+        self.assertEqual(len(data), handle.tell())
+
     def test_write_post(self):
         session = self._create_mock_session()
         handle = rw_handles.VmdkWriteHandle(session, '10.1.2.3', 443,
@@ -263,17 +288,31 @@ class VmdkWriteHandleTest(base.TestCase):
 
         session.invoke_api = mock.Mock(
             side_effect=session_invoke_api_side_effect)
+        handle._get_progress = mock.Mock(return_value=100)
         handle.close()
         self.assertEqual(2, session.invoke_api.call_count)
 
+    def test_get_vm_incomplete_transfer(self):
+        session = self._create_mock_session()
+        handle = rw_handles.VmdkWriteHandle(session, '10.1.2.3', 443, 'rp-1',
+                                            'folder-1', None, 100)
+
+        handle._get_progress = mock.Mock(return_value=99)
+        session.invoke_api = mock.Mock()
+
+        self.assertRaises(exceptions.ImageTransferException,
+                          handle.get_imported_vm)
+
 
 class VmdkReadHandleTest(base.TestCase):
     """Tests for VmdkReadHandle."""
 
     def setUp(self):
-        super(VmdkReadHandleTest, self).setUp()
+        super().setUp()
+
+    def _mock_connection(self, read_data='fake-data'):
         self._resp = mock.Mock()
-        self._resp.read.return_value = 'fake-data'
+        self._resp.read.return_value = read_data
         self._conn = mock.Mock()
         self._conn.getresponse.return_value = self._resp
         patcher = mock.patch(
@@ -282,7 +321,9 @@ class VmdkReadHandleTest(base.TestCase):
         HTTPConnectionMock = patcher.start()
         HTTPConnectionMock.return_value = self._conn
 
-    def _create_mock_session(self, disk=True, progress=-1):
+    def _create_mock_session(self, disk=True, progress=-1,
+                             read_data='fake-data'):
+        self._mock_connection(read_data=read_data)
         device_url = mock.Mock()
         device_url.disk = disk
         device_url.url = 'http://*/ds/disk1.vmdk'
@@ -303,7 +344,7 @@ class VmdkReadHandleTest(base.TestCase):
         vim_cookie = mock.Mock()
         vim_cookie.name = 'name'
         vim_cookie.value = 'value'
-        session.vim.client.options.transport.cookiejar = [vim_cookie]
+        session.vim.client.cookiejar = [vim_cookie]
         return session
 
     def test_init_failure(self):
@@ -323,8 +364,30 @@ class VmdkReadHandleTest(base.TestCase):
         handle = rw_handles.VmdkReadHandle(session, '10.1.2.3', 443,
                                            'vm-1', '[ds] disk1.vmdk',
                                            chunk_size * 10)
+        fake_data = 'fake-data'
         data = handle.read(chunk_size)
-        self.assertEqual('fake-data', data)
+        self.assertEqual(fake_data, data)
+        self.assertEqual(len(fake_data), handle._bytes_read)
+
+    def test_read_small(self):
+        read_data = 'fake'
+        session = self._create_mock_session(read_data=read_data)
+
+        read_size = len(read_data)
+        handle = rw_handles.VmdkReadHandle(session, '10.1.2.3', 443,
+                                           'vm-1', '[ds] disk1.vmdk',
+                                           read_size * 10)
+        handle.read(read_size)
+        self.assertEqual(read_size, handle._bytes_read)
+
+    def test_tell(self):
+        chunk_size = rw_handles.READ_CHUNKSIZE
+        session = self._create_mock_session()
+        handle = rw_handles.VmdkReadHandle(session, '10.1.2.3', 443,
+                                           'vm-1', '[ds] disk1.vmdk',
+                                           chunk_size * 10)
+        data = handle.read(chunk_size)
+        self.assertEqual(len(data), handle.tell())
 
     def test_update_progress(self):
         chunk_size = len('fake-data')
@@ -351,6 +414,7 @@ class VmdkReadHandleTest(base.TestCase):
 
         session.invoke_api = mock.Mock(
             side_effect=session_invoke_api_side_effect)
+        handle._get_progress = mock.Mock(return_value=100)
         handle.close()
         self.assertEqual(2, session.invoke_api.call_count)
 
@@ -372,7 +436,7 @@ class ImageReadHandleTest(base.TestCase):
         max_items = 10
         item = [1] * 10
 
-        class ImageReadIterator(six.Iterator):
+        class ImageReadIterator:
 
             def __init__(self):
                 self.num_items = 0
